#!/usr/bin/env python
"""
_SetFWJRPath_

Oracle implementation of Jobs.SetFWJRPath
"""




from WMCore.WMBS.MySQL.Jobs.SetFWJRPath import SetFWJRPath as MySQLSetFWJRPath

class SetFWJRPath(MySQLSetFWJRPath):
    """
    Identical to MySQL version.
    """
    pass
