#!/usr/bin/env python
"""
_RetriedJobs_

Oracle implementation of Workflow.FailedJobs
"""




from WMCore.WMBS.MySQL.Workflow.RetriedJobs import RetriedJobs as MySQLRetriedJobs

class RetriedJobs(MySQLRetriedJobs):
    pass
