#!/usr/bin/env python
"""
_LoadOutput_

Oracle implementation of Workflow.LoadOutput
"""

from WMCore.WMBS.MySQL.Workflow.LoadOutput import LoadOutput as LoadOutputMySQL

class LoadOutput(LoadOutputMySQL):
    pass
