#!/usr/bin/env python
"""
_DBSBuffer.UpdateAlgo_

Add PSetHash to Algo in DBS Buffer
"""




from WMComponent.DBS3Buffer.MySQL.UpdateAlgo import UpdateAlgo as MySQLUpdateAlgo

class UpdateAlgo(MySQLUpdateAlgo):
    """
    _DBSBuffer.UpdateAlgo_

    Add PSetHash to Algo in DBS Buffer
    """
    pass
