@CHARSET "UTF-8";

.masthead {
	background:rgb(215, 215, 215);
	border:solid grey 0px;
	border-bottom-width: 0.01em;
}
.masthead .mastheadLeft ul, .masthead .mastheadRight ul {
	border:solid rgb(215, 215, 215) 0px;
	color: grey;
}
.masthead .mastheadLeft ul li a, .masthead .mastheadRight ul li a {
	color:grey;
}
.masthead #help a{
	color:white;
	background:rgb(150, 0, 0);
}
/* This next bit is for site dependant masthead */
.mastheadfooter {
	background:rgb(150, 0, 0);
}
.mastheadfooter ul{
	color:white;
}
.mastheadfooter ul li {
	color:white;
}
.mastheadfooter ul li a, .mastheadfooter ul li a:visited {
	color:white;
}
.mastheadfooter ul li a:hover {
	color:white;
}
.mastheadfooter ul li a:active {
	color:white;
}