"""
_GetDeletableBlocks_

Oracle implementation of PhEDExInjector.GetDeletableBlocks

"""

from __future__ import division
from __future__ import print_function

from WMComponent.RucioInjector.Database.MySQL.GetDeletableBlocks import GetDeletableBlocks as MySQLBase


class GetDeletableBlocks(MySQLBase):
    pass
