#!/usr/bin/env python
"""
_DeleteCheck_

Oracle implementation of DeleteCheckFiles
"""

from WMCore.WMBS.MySQL.Files.DeleteCheck import DeleteCheck as MySQLDeleteCheck

class DeleteCheck(MySQLDeleteCheck):
    pass
