# -*- coding: utf-8 -*-
"""
_GetJobSlots_

Oracle implementation of Locations.GetJobSlots
Created on Mon Jun 18 15:28:46 2012

@author: dballest
"""

from WMCore.WMBS.MySQL.Locations.GetJobSlots import GetJobSlots as MySQLGetJobSlots

class GetJobSlots(MySQLGetJobSlots):
    """
    _GetJobSlots_

    Same as MySQL version
    """
    pass
